@@ -19,9 +19,11 @@ from random import choice
 import string
 import os
 from time import sleep
+import datetime
 import re
 import sys
 import operator
+import json
 from shutil import copyfile, copytree
 from subprocess import Popen, PIPE
 from itertools import *
@@ -34,8 +36,8 @@ from calculate.lib.utils.files import (isMount, removeDir,
                                        listDirectory, checkUtils,
                                        MAGIC_COMPRESS, MAGIC_SYMLINK,
                                        MAGIC_CONTINUE, makeDirectory,
-                                       isEmpty, check_rw,
-                                       PercentProgress, getProgPath)
+                                       isEmpty, check_rw, calculate_exclude,
+                                       PercentProgress, getProgPath, xztaropen)
 from calculate.lib.utils.device import (detectDeviceForPartition,
                                         getUdevDeviceInfo, countPartitions)
 from calculate.lib.utils.tools import classproperty, Signal, traverse
@@ -174,11 +176,16 @@ class Distributive(object):
                 return IsoDistributive(filename)
             elif "7-zip" in ftype or \
                             "POSIX tar archive" in ftype:
-                return ArchiveDistributive(filename)
+                if "rootfs.tar" in filename:
+                    return ContainerDistributive(path.dirname(filename))
+                else:
+                    return ArchiveDistributive(filename)
             elif "Squashfs filesystem" in ftype:
                 return SquashDistributive(filename)
             elif path.isdir(filename):
-                if path.isfile(path.join(filename, "livecd")):
+                if path.isfile(path.join(filename, "rootfs.tar.xz")):
+                    return ContainerDistributive(filename)
+                elif path.isfile(path.join(filename, "livecd")):
                     return IsoDistributive(filename)
                 else:
                     important_dirs = ("etc", "lib", "bin", "sbin", "var")
@@ -655,7 +662,7 @@ class DirectoryDistributive(Distributive):
         self.system_mounted = False
 
     def getType(self):
-        return _("directory '%s'") % self.directory
+        return _("directory '%s'") % _u8(self.directory)
 
     def bindDirectory(self, mdirectory):
         for child in self.childs:
@@ -1268,6 +1275,8 @@ class ArchiveDistributive(Distributive):
             return "gzip"
         elif "7-zip archive data" in p_file.read():
             return "7z"
+        elif "XZ compressed data" in p_file.read():
+            return "xz"
         elif file and file.endswith(".tar.lzma"):
             if path.exists('/usr/bin/7za'):
                 return "7z"
@@ -1300,6 +1309,9 @@ class ArchiveDistributive(Distributive):
         elif archiveType == "gzip":
             arch_cmd = getProgPath('/bin/gzip')
             params = ["-dc"]
+        elif archiveType == "xz":
+            arch_cmd = getProgPath('/bin/xz')
+            params = ["-dc"]
         else:
             raise DistributiveError(_("Unknown archive type '%s'") %
                                     archfile)
@@ -1375,13 +1387,91 @@ class ArchiveDistributive(Distributive):
         return ld
 
 
+class ContainerDistributive(ArchiveDistributive):
+    def __init__(self, basedir, parent=None, exclude=None,
+                 include=None,
+                 mdirectory="/var/calculate/tmp/stage"):
+        Distributive.__init__(self, parent=parent)
+        self.basedirectory = basedir
+        self.file = path.join(basedir, "rootfs.tar.xz")
+        self.meta = path.join(basedir, "meta.tar.xz")
+        self.lxd = path.join(basedir, "lxd.tar.xz")
+        self.exclude = [] if not exclude else exclude
+        self.include = [] if not include else include
+        self.mdirectory = mdirectory
+
+    def mtime2build(self, fn):
+        if path.exists(fn):
+            build_time = datetime.datetime.fromtimestamp(os.stat(fn).st_mtime)
+            return build_time.strftime("%Y%m%d")
+        return ""
+
+    def packToArchive(self, directory, file):
+        if not path.exists(self.basedirectory):
+            self._makeDirectory(self.basedirectory)
+        tar_command = getProgPath("/usr/bin/tar")
+        params = ["cf", file, "-J", "-C", directory]
+        if self.exclude:
+            exclude_list = list(calculate_exclude(
+                directory, exclude=self.exclude, include=self.include))
+            params += map(lambda x: "--exclude=./%s" % x, exclude_list)
+        params += ["."]
+
+        #debug_file = "/var/calculate/tmp/rootfs.tar.xz"
+        #if path.exists(debug_file):
+        #    p = process("/bin/cp", debug_file, file)
+        #else:
+        p = process(tar_command, *params, stderr=process.STDOUT)
+        try:
+            if p.failed():
+                raise DistributiveError(_("Failed to create the archive") +
+                                        " '%s':\n%s" % (file, p.read()))
+        except BaseException:
+            removeDir(self.basedirectory)
+            raise
+
+    def get_information(self):
+        if path.exists(self.lxd):
+            with xztaropen(self.lxd) as f:
+                try:
+                    metadata_yaml = f.getmember("metadata.yaml")
+                    metadata = json.load(f.extractfile(metadata_yaml))
+                    return {
+                        'os_linux_build':
+                            metadata["calculate"]['os_linux_build'],
+                        'os_arch_machine':
+                            metadata["calculate"]['os_arch_machine'],
+                        'os_linux_shortname':
+                            metadata["calculate"]['os_linux_shortname'],
+                        'os_linux_subname':
+                            metadata["calculate"]['os_linux_subname'],
+                        'cl_profile_name':
+                            metadata["calculate"]['cl_profile_name'],
+                        'os_linux_name':
+                            metadata["calculate"]['os_linux_name'],
+                        'os_linux_ver':
+                            metadata["calculate"].get('os_linux_ver', "17"),
+                    }
+                except (KeyError, ValueError) as e:
+                    pass
+        return {
+            'os_linux_build': self.mtime2build(self.file),
+            'os_arch_machine': "x86_64",
+            'os_linux_shortname': 'Container',
+            'os_linux_subname': "",
+            'cl_profile_name': "",
+            'os_linux_name': _('Unknown Container')
+        }
+
+
 class SquashDistributive(Distributive):
     def __init__(self, file, parent=None, mdirectory=None, exclude=None,
-                 compress=""):
+                 compress="", include=None):
         Distributive.__init__(self, parent=parent)
         self.file = file
         self.mdirectory = mdirectory or DefaultMountPath.SquashImage
         self.exclude = exclude or []
+        self.include = include or []
         self.compress = compress if compress and compress != "gzip" else ""
 
     def getType(self):
@@ -1424,7 +1514,9 @@ class SquashDistributive(Distributive):
             cmd += ["-comp", self.compress]
         cmd += ["-progress"]
         if self.exclude:
-            cmd += ["-e"] + self.exclude
+            exclude_list = list(calculate_exclude(directory, exclude=self.exclude,
+                                include=self.include))
+            cmd += ["-e"] + exclude_list
         # возможность использовать заранее подготовленный livecd.squashfs
         if path.exists('/var/calculate/tmp/livecd.squashfs'):
             os.system('cp -L /var/calculate/tmp/livecd.squashfs %s' % file)
@@ -1467,7 +1559,7 @@ class SquashDistributive(Distributive):
 class IsoDistributive(Distributive):
     def __init__(self, file, parent=None, mdirectory=None,
                  bdirectory=None, exclude=None, compress="gzip",
-                 vol_id="CALCULATE"):
+                 vol_id="CALCULATE", include=None):
         if bdirectory is None:
             bdirectory = DefaultMountPath.BuildDirectory
         Distributive.__init__(self, parent=parent)
@@ -1482,6 +1574,7 @@ class IsoDistributive(Distributive):
         else:
             self.bdirectory = self._getMntDirectory(bdirectory)
         self.exclude = [] if not exclude else exclude
+        self.include = [] if not include else include
         self.compress = compress
         self.eventPrepareIso = Signal()
 
@@ -1544,7 +1637,8 @@ class IsoDistributive(Distributive):
                                     self.file)
         return SquashDistributive(path.join(mdirectory, fileLive),
                                   parent=self, exclude=self.exclude,
-                                  compress=self.compress)
+                                  compress=self.compress,
+                                  include=self.include)
 
     def getIsoContentDirectory(self):
         """Return directory with content of iso image"""
@@ -1647,6 +1741,7 @@ class IsoDistributive(Distributive):
                 squashDistr = SquashDistributive(
                     path.join(isoDirectory, liveimage),
                     exclude=self.exclude,
+                    include=self.include,
                     compress=self.compress)
                 squashDistr.installFrom(distDirectory, **kwargs)
             # prepare iso
