@@ -281,6 +281,22 @@ class Install(MethodsInterface):
             "--efi-directory=%s" %
             target.getEfiDirectory(),
             "--force"]
+        # проверяем наличие в nv-ram нужной нам записи для исключения повтора
+        efi_boot_mgr = getProgPath('/usr/sbin/efibootmgr')
+        efi_disk = self.clVars.Select("os_install_disk_dev",
+                                      where="os_install_disk_mount",
+                                      eq="/boot/efi", limit=1)
+        if efi_disk:
+            efi_uuid = getUdevDeviceInfo(
+                name=efi_disk).get("ID_PART_ENTRY_UUID", "")
+            if efi_uuid:
+                p_efibootmgr = process(efi_boot_mgr, "-v")
+                data = p_efibootmgr.read()
+                if re.search(r"Boot.*calculate.*GPT,{uuid}.*{efipath}".format(
+                        uuid=efi_uuid,
+                        efipath=r"\\EFI\\calculate\\grubx64.efi"),
+                        data, flags=re.M | re.I):
+                    grub_params.append("--no-nvram")
         # в случае установки на usb-hdd EFI загрузчик не прописывается
         # в efivars
         if self.clVars.Get('os_install_root_type') == 'usb-hdd':
@@ -298,7 +314,6 @@ class Install(MethodsInterface):
         # если среди загрузочных записей отсутствует запись
         # calculate и dmesg содержит сообщение об ошибке efivars -
         # запись создать не удалось
-        efi_boot_mgr = getProgPath('/usr/sbin/efibootmgr')
         dmesg = getProgPath('/bin/dmesg')
         if efi_boot_mgr and dmesg:
             if not re.search('Boot.*calculate',
